import tkinter as tk
from tkinter import messagebox, filedialog, scrolledtext
import socket
import json
import os

###################################################################################################################
# This is the configuration file which is loaded at startup. If it does not exist, it will be created. Optional.
###################################################################################################################
CONFIG_FILE = os.path.join(os.path.expanduser("~"), 'config.json')  

##################################################################################################################
# Main application class inheriting from tkinter.Tk
##################################################################################################################
class XMLClientApp(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title("XIQS Tester App Python version")  
        self.geometry("800x600")  # Set window size
        self.config = self.load_config()                                            # Load configuration from file
        self.create_widgets()

##################################################################################################################
# Method to create and layout UI widgets
##################################################################################################################
    
    def create_widgets(self):
        self.grid_columnconfigure(1, weight=1)
        self.grid_columnconfigure(2, weight=1)
        self.grid_rowconfigure(6, weight=1)
        self.grid_rowconfigure(8, weight=1)
        self.grid_rowconfigure(10, weight=1)

##################################################################################################################
# Field for connection information
##################################################################################################################
        
        tk.Label(self, text="IP address:").grid(row=0, column=0, sticky=tk.W, padx=5, pady=5)
        self.ip_entry = tk.Entry(self)
        self.ip_entry.grid(row=0, column=1, padx=5, pady=5, sticky=tk.EW)
        self.ip_entry.insert(0, self.config.get('ip', ''))  # Load saved IP address

        tk.Label(self, text="Port:").grid(row=1, column=0, sticky=tk.W, padx=5, pady=5)
        self.port_entry = tk.Entry(self)
        self.port_entry.grid(row=1, column=1, padx=5, pady=5, sticky=tk.EW)
        self.port_entry.insert(0, self.config.get('port', ''))  # Load saved port

        tk.Label(self, text="Login:").grid(row=2, column=0, sticky=tk.W, padx=5, pady=5)
        self.login_entry = tk.Entry(self)
        self.login_entry.grid(row=2, column=1, padx=5, pady=5, sticky=tk.EW)
        self.login_entry.insert(0, self.config.get('login', ''))  # Load saved login

        tk.Label(self, text="Password:").grid(row=3, column=0, sticky=tk.W, padx=5, pady=5)
        self.password_entry = tk.Entry(self, show='*')
        self.password_entry.grid(row=3, column=1, padx=5, pady=5, sticky=tk.EW)
        self.password_entry.insert(0, self.config.get('password', ''))  # Load saved password

        tk.Label(self, text="Timeout (s):").grid(row=4, column=0, sticky=tk.W, padx=5, pady=5)
        self.timeout_entry = tk.Entry(self)
        self.timeout_entry.grid(row=4, column=1, padx=5, pady=5, sticky=tk.EW)
        self.timeout_entry.insert(0, self.config.get('timeout', '5'))  # Load saved timeout

##################################################################################################################
# Checkbox for auto-login
##################################################################################################################
        self.autologin_var = tk.BooleanVar(value=self.config.get('autologin', False))
        self.autologin_check = tk.Checkbutton(self, text="Auto-login", variable=self.autologin_var)
        self.autologin_check.grid(row=5, column=1, sticky=tk.W, padx=5, pady=5)

##################################################################################################################
# Text area for XML input
##################################################################################################################
        tk.Label(self, text="XML:").grid(row=6, column=0, sticky=tk.NW, padx=5, pady=5)
        self.xml_text = scrolledtext.ScrolledText(self, width=40, height=10)
        self.xml_text.grid(row=6, column=1, padx=5, pady=5, sticky=tk.NSEW)

        self.send_button = tk.Button(self, text="Send", command=self.send_request)
        self.send_button.grid(row=7, column=1, sticky=tk.E, padx=5, pady=5)

##################################################################################################################
# Text area for server response
##################################################################################################################
        
        tk.Label(self, text="Response:").grid(row=8, column=0, sticky=tk.NW, padx=5, pady=5)
        self.response_text = scrolledtext.ScrolledText(self, width=40, height=10)
        self.response_text.grid(row=8, column=1, padx=5, pady=5, sticky=tk.NSEW)

##################################################################################################################
# Frame for flush buttons
##################################################################################################################
        button_frame = tk.Frame(self)
        button_frame.grid(row=9, column=1, padx=5, pady=5, sticky=tk.EW)
        button_frame.grid_columnconfigure(0, weight=1)
        button_frame.grid_columnconfigure(1, weight=1)

        self.clear_request_button = tk.Button(button_frame, text="Flush request", command=self.clear_request)
        self.clear_request_button.grid(row=0, column=0, padx=5, pady=5, sticky=tk.EW)

        self.clear_response_button = tk.Button(button_frame, text="Flush response", command=self.clear_response)
        self.clear_response_button.grid(row=0, column=1, padx=5, pady=5, sticky=tk.EW)

##################################################################################################################
# Button to open XML directory
##################################################################################################################
        self.select_dir_button = tk.Button(self, text="Open XML directory", command=self.select_directory)
        self.select_dir_button.grid(row=0, column=2, padx=5, pady=5, sticky=tk.EW)

##################################################################################################################
# Frame for listing XML files
##################################################################################################################
        self.file_list_frame = tk.Frame(self)
        self.file_list_frame.grid(row=1, column=2, rowspan=8, padx=5, pady=5, sticky=tk.NSEW)
        self.file_list_frame.grid_rowconfigure(0, weight=1)
        self.file_list_frame.grid_columnconfigure(0, weight=1)

        self.file_list_canvas = tk.Canvas(self.file_list_frame)
        self.file_list_scrollbar = tk.Scrollbar(self.file_list_frame, orient="vertical", command=self.file_list_canvas.yview)
        self.file_list_scrollable_frame = tk.Frame(self.file_list_canvas)

        self.file_list_scrollable_frame.bind(
            "<Configure>",
            lambda e: self.file_list_canvas.configure(
                scrollregion=self.file_list_canvas.bbox("all")
            )
        )

        self.file_list_canvas.create_window((0, 0), window=self.file_list_scrollable_frame, anchor="nw")
        self.file_list_canvas.configure(yscrollcommand=self.file_list_scrollbar.set)

        self.file_list_canvas.pack(side="left", fill="both", expand=True)
        self.file_list_scrollbar.pack(side="right", fill="y")

##################################################################################################################
# Mouse wheel for scrolling
##################################################################################################################
        self.file_list_canvas.bind_all("<MouseWheel>", self._on_mouse_wheel) 
        self.file_checkbuttons = []

##################################################################################################################
# Method to handle mouse wheel scrolling in the file list
##################################################################################################################
    def _on_mouse_wheel(self, event):
        self.file_list_canvas.yview_scroll(int(-1*(event.delta/120)), "units")

##################################################################################################################
# Method to load configuration from file
##################################################################################################################
    def load_config(self):
        try:
            if os.path.exists(CONFIG_FILE):                     # Test if configuration file exist
                with open(CONFIG_FILE, 'r') as file:            # Open configuration file in read mode
                    return json.load(file)                      # Return the loaded configuration from file
            else:                                               # the configuration file does not exist
                with open(CONFIG_FILE, 'w') as file:            # Create an empty file 
                    json.dump({}, file)                         # Use Json method to save an empty json configuration file
                return {}                                       # Return empty Json
        except PermissionError as e:
            messagebox.showerror("Error", f"Permission denied for: {CONFIG_FILE}: {str(e)}")    # Error if permission denied to read or write file
            return {}                                           # Return empty Json


##################################################################################################################
# Method to save current configuration to file
##################################################################################################################
    def save_config(self):
        config = {                                              # Json object for configuration
            'ip': self.ip_entry.get(),
            'port': self.port_entry.get(),
            'login': self.login_entry.get(),
            'password': self.password_entry.get(),
            'timeout': self.timeout_entry.get(),
            'autologin': self.autologin_var.get()
        }
        try:
            with open(CONFIG_FILE, 'w') as file:
                json.dump(config, file)
        except PermissionError as e:
            messagebox.showerror("Error", f"Permission denied for: {CONFIG_FILE}: {str(e)}")

##################################################################################################################
# Method to clear the XML request text area
##################################################################################################################
    def clear_request(self):
        self.xml_text.delete("1.0", tk.END)

##################################################################################################################
# Method to clear the response text area
##################################################################################################################
    def clear_response(self):
        self.response_text.delete("1.0", tk.END)

##################################################################################################################
# Method to send XML request to the server
##################################################################################################################
    def send_request(self):
        ip = self.ip_entry.get()                                # All connection informations to open socket between client and server
        port = int(self.port_entry.get())
        login = self.login_entry.get()
        password = self.password_entry.get()
        xml_data = self.xml_text.get("1.0", tk.END).strip()
        timeout = int(self.timeout_entry.get())
        autologin = self.autologin_var.get()

        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:    # Create socket with AF_INET (internet adress family for IPv4) and SOCK_STREAM for TCP protocol
                s.settimeout(timeout)                                       # Set timeout to avoid possible infinite loop 
                s.connect((ip, port))                                       # Init the conection
                # Test if autologin enabled. It's recommanded to keep this ON to automaticaly send Login command
                # Hard coded login command. Can be loaded from other means if necessary
                if autologin:                                               
                    login_xml = f'''<?xml version="1.0" encoding="UTF-8"?>  
<command name="loginXIQS">
  <param type="string">{login}</param>
  <param type="string">{password}</param>
</command>'''
                    
                    s.sendall(login_xml.encode('utf-8'))                                                    # Socket send login command
                    login_response = s.recv(4096).decode('utf-8')                                           # Socket decode response server as text
                    self.response_text.delete("1.0", tk.END)                                                # Auto flush text area
                    self.response_text.insert(tk.END, "Auto-login response:\n" + login_response + "\n\n")   # Set text server response into text area
                                                                                                            # Please note this is just raw response. If login command fail, the error is not catched

                s.sendall(xml_data.encode('utf-8'))                                                         # After login socket send choosen request to server
                response = s.recv(4096).decode('utf-8')                                                     # Socket decode response server as text

            self.response_text.insert(tk.END, response)                                                     # Set text server response into text area
            self.save_config()                                                                              # Save the configuration after sending request
        except Exception as e:
            messagebox.showerror("Error", f"An error occurred: {str(e)}")                                   # If any error, show it

##################################################################################################################
# Method to select a directory containing XML files
##################################################################################################################
    def select_directory(self):
        directory = filedialog.askdirectory()
        if directory:
            self.display_files(directory)

##################################################################################################################
# Method to display XML files from the selected directory
##################################################################################################################
    def display_files(self, directory):
        for widget in self.file_list_scrollable_frame.winfo_children():                                     # Destory all widget button at xml file loadins
            widget.destroy()

        self.file_checkbuttons.clear()                                                                      # Flush button list

        for file_name in os.listdir(directory):                                                             # Parse all xml file in choosen directory
            if file_name.endswith(".xml"):                                                                  # if file is not xml, it will be ignored
                button = tk.Button(self.file_list_scrollable_frame, text=file_name, command=lambda fn=file_name: self.load_xml_file(directory, fn))
                button.pack(anchor='w', fill='x', padx=5)                                                   # Button creation
                self.file_checkbuttons.append(button)                                                       # Appen buttons to list

##################################################################################################################
# Method to load selected XML file content into the XML text area
##################################################################################################################
    def load_xml_file(self, directory, file_name):
        file_path = os.path.join(directory, file_name)
        with open(file_path, 'r') as file:
            xml_content = file.read()
            self.xml_text.delete("1.0", tk.END)
            self.xml_text.insert(tk.END, xml_content)

##################################################################################################################
# Entry point for the application
##################################################################################################################
if __name__ == "__main__":
    app = XMLClientApp()                                                                                     # Create an instance of the application
    app.mainloop()                                                                                           # Run the application's main loop
